/* IRCfs - IRC FileServ for *nix.
 * Copyright (C) 2002  Nick 'Zaf' Clifford <zaf@nrc.co.nz>
 * For licensing details, refer to the LICENSE file in the source 
 * code directory.
 *
 * The trace system is for internal debugging and diagnostics of the
 * core system. Unlike the debug system, it is not setup on runtime
 * and is not very configurable.
 *
 * Each module must be registered in the header file.
 *
 * By default the output of the trace system goes to STDERR. 
 *
 * Since this system incurrs a somewhat large overhead in runtime speeds,
 * it is a compile-time setting. Simply undefine ENABLE_TRACE to disable
 * the system. (Should be done in ./configure script).
 *
 * For examples of how to use, rtfs.
 */
#ifndef __include_trace_h__
#define __include_trace_h__

#include <stdarg.h>
/* The standard configure header */
#include "config.h"

enum trace_levels {
	T_ALL   = 0xFF,/* All traces */
	T_FLOW  = 0x01,/* Flow of code */
	T_FUNC  = 0x02,/* Function entry/exit */
	T_COND  = 0x04,/* Conditions/checks */
	T_CHECK = 0x08,/* Checkpoints */
	T_NOTE  = 0x10,/* Important Notes/Significant events */
	T_WARN  = 0x12,/* Warnings */
	T_ERR   = 0x14,/* Errors */
	T_ASSERT= 0x18,/* Assertions fail */
	D_NONE  = 0x0  /* No traces */
};

enum trace_systems {
	T_OBJ	= 0x00000001, /* Object code */
	T_MEM	= 0x00000002, /* Memory code */
	T_INT	= 0x00000004, /* Internal code */
	T_DEBUG = 0x00000008, /* Debugging code */
	T_EVENT = 0x00000010, /* Event code */
	T_SYS_NONE = 0x0
};


#ifdef ENABLE_TRACE

#define tracev(sys,lvl,format, va) __tracev(sys,lvl,__LINE__,__FILE__,__FUNCTION__,format, va)

#define trace(sys,lvl,format, ...) __trace(sys,lvl,__LINE__,__FILE__,__FUNCTION__,format, ## __VA_ARGS__)
#else
#define tracev(sys,lvl,format,va) do {}while(0)
#define trace(sys,lvl,format,...) do {}while(0)
#endif
void __trace(unsigned long sys, unsigned long lvl, 
		int line, const char *file, const char *func,
		const char *format, ...);


void __tracev(unsigned long sys, unsigned long lvl, 
		int line, const char *file, const char *func,
		const char *format, va_list vl);

void trace_off();
void trace_on(unsigned long lvl);

void trace_sys_on(unsigned long sys);
void trace_sys_off();

void trace_output_fd(int fd);

#define TRACE_DEFAULT_LVL	T_ALL

#endif
